@@ -1,7 +1,7 @@
 # ABSTRACT: Utilities for Monitoring ElasticSearch
 package App::ElasticSearch::Utilities;
 
-our $VERSION = '3.5'; # VERSION
+our $VERSION = '3.6'; # VERSION
 
 use strict;
 use warnings;
@@ -30,6 +30,7 @@ use Sub::Exporter -setup => {
         es_indices
         es_indices_meta
         es_index_valid
+        es_index_strip_date
         es_index_days_old
         es_index_shards
         es_index_segments
@@ -211,7 +212,7 @@ sub es_request {
     $options->{$_} = $instance->{$_} for qw(host port);
 
     $options->{method} ||= 'GET';
-    $options->{body} = $body if defined $body && ref $body eq 'HASH';
+    $options->{body} = $body if defined $body;
     $options->{command} = $url;
     my $index = 'NoIndex';
 
@@ -263,7 +264,7 @@ sub es_request {
 my %_nodes;
 sub es_nodes {
     if(!keys %_nodes) {
-        my $res = es_request('_cluster/state', {
+        my $res = es_request('_cat/state', {
             uri_param => {
                 filter_nodes         => 0,
                 filter_routing_table => 1,
@@ -289,13 +290,7 @@ my $_indices_meta;
 sub es_indices_meta {
 
     if(!defined $_indices_meta) {
-        my $result = es_request('_cluster/state', {
-            uri_param => {
-                filter_routing_table => 1,
-                filter_nodes         => 1,
-                filter_blocks        => 1,
-            },
-        });
+        my $result = es_request('_cluster/state/metadata');
         $_indices_meta = $result->{metadata}{indices};
         if ( !defined $_indices_meta ) {
             output({stderr=>1,color=>"red"}, "es_indices(): Unable to locate indices in status!");
@@ -380,6 +375,18 @@ sub es_indices {
 }
 
 
+sub es_index_strip_date {
+    my ($index) = @_;
+
+    return -1 unless defined $index;
+
+    if( my ($indexName) = ($index =~ /^(.*)[-_]$PATTERN_REGEX{DATE}$/) ) {
+        return $indexName;
+    }
+    return undef;
+}
+
+
 my $NOW = timelocal(0,0,0,(localtime)[3,4,5]);
 sub es_index_days_old {
     my ($index) = @_;
@@ -590,7 +597,7 @@ App::ElasticSearch::Utilities - Utilities for Monitoring ElasticSearch
 
 =head1 VERSION
 
-version 3.5
+version 3.6
 
 =head1 SYNOPSIS
 
@@ -696,6 +703,10 @@ Default is 1, set to 0 to disable checking index age.
 
 =back
 
+=head2 es_index_strip_date( 'index-name' )
+
+Returns the index name with the date removed.
+
 =head2 es_index_days_old( 'index-name' )
 
 Return the number of days old this index is.
