@@ -99,7 +99,7 @@ for ( @metrics ) {
         verbose($_);
     }
     else {
-        output($_);
+        output({data=>1},$_);
     }
 }
 
@@ -192,7 +192,7 @@ es-graphite-dynamic.pl - Dynamically gather metrics and send to graphite
 
 =head1 VERSION
 
-version 3.5
+version 3.6
 
 =head1 SYNOPSIS
 
